//
//  IDmeVerify.h
//  ID.me Verify
//
//  Created by Arthur Sabintsev on 7/16/13.
//  Copyright (c) 2013 ID.me, Inc. All rights reserved.
//

// Apple Frameworks
#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

// ID.me Verify - Scannable Credentials
#import "IDmeCredential.h"
#import "IDmeStateLicense.h"
#import "IDmeCommonAccessCard.h"
#import "IDmeUniformedServicesCard.h"
#import "IDmeUniformedServicesDependentCard.h"

// ID.me Verify - Console Logging Macro
#define IDmeLog(fmt, ...)                   NSLog((@"[ID.me Verify]: " fmt), ##__VA_ARGS__)
#define IDME_VERIFY_ERROR_DOMAIN            @"ID.me Verify Error Domain"

/// This typedef is used to configure the types of credentials can be scanned. \nThe values are bitshifted for your convenience.
typedef NS_OPTIONS(NSUInteger, IDmeVerifyDetectCredentialTypes)
{
    IDmeVerifyDetectCredentialTypeAll                               = (0x1 << 0), // 0x00000001
    IDmeVerifyDetectCredentialTypeStateLicense                      = (0x1 << 1), // 0x00000010
    IDmeVerifyDetectCredentialTypeCommonAccessCard                  = (0x1 << 2), // 0x00000100
    IDmeVerifyDetectCredentialTypeUniformedServicesCard             = (0x1 << 3), // 0x00001000
    IDmeVerifyDetectCredentialTypeUniformedServicesDependentCard    = (0x1 << 4)  // 0x00010000
};

/// The installed version of the ID.me Verify SDK.
extern NSString *const kIDmeVerifySDKVersion;

/// References the @c affiliation key in the JSON object.
extern NSString *const kIDmeVerifyKeyAffiliation;

/// References the @c ID key in the JSON object.
extern NSString *const kIDmeVerifyKeyID;

/// References the @c Verified key in the JSON object.
extern NSString *const kIDmeVerifyKeyVerified;

/// A string value of @b Not @b Available may on occassion be returned for the @c barcode, @c SSN, & @c EDIPI values.
extern NSString *const kIDmeVerifyErrorNotAvailable;

/// The plain text error description returned for the @c IDmeVerifyErrorCodeScannerAccessForbidden type in the @c IDmeVerifyErrorCode typedef.
extern NSString *const kIDmeVerifyErrorScannerAccessForbidden;

/// The plain text error description returned for the @c IDmeVerifyErrorCodeScannerAccessForbidden type in the @c IDmeVerifyErrorCode typedef.
extern NSString *const kIDmeVerifyErrorNoScanPerformed;

/// The plain text error description returned for the @c IDmeVerifyErrorCodeCouldNotParseCredential type in the @c IDmeVerifyErrorCode typedef.
extern NSString *const kIDmeVerifyErrorCouldNotParseCredential;

/// The plain text error description returned for the @c IDmeVerifyErrorCodeCredentialOwnerIsUnderLegalAge type in the @c IDmeVerifyErrorCode typedef.
extern NSString *const kIDmeVerifyCredentialOwnerIsUnderLegalAge;

/// The plain text error description returned for the @c IDmeVerifyErrorCodeVerificationWasCanceledByUser type in the @c IDmeVerifyErrorCode typedef.
extern NSString *const kIDmeVerifyVerificationWasCanceledByUser;

/// The results block that returns data and errors (if they exist) when scanning and parsing a credential.
typedef void (^IDmeVerifyScanResults)(IDmeCredential *credential, NSError *error);

/// The results block that returns data and errors (if they exist) when performing a verification.
typedef void (^IDmeVerifyVerificationResults)(NSDictionary *userProfile, NSError *error);

/// This typedef differentiates errors that may occur when using the credential scanner and parser.
typedef NS_ENUM(NSUInteger, IDmeVerifyErrorCode)
{
    /// Error occurs if developer attempts to access scanner, but is not given access to it. To gain access to scanner, send an email to @b mobile@id.me.
    IDmeVerifyErrorCodeScannerAccessForbidden = 101,
    
    /// Error occurs if user exits scanner without scanning a credential.
    IDmeVerifyErrorCodeNoScanPerformed,
    
    /// Error occurs if scanned credential could not be parsed.
    IDmeVerifyErrorCodeCouldNotParseCredential,
    
    /// Error occurs if owner of scanned credential is under age.
    IDmeVerifyErrorCodeCredentialOwnerIsUnderLegalAge,
    
/*!
 * Error occurs if user succesfully verified their group affiliation, but there was a problem with the user's profile being returned. 
 * This should never occur, but this error was added to handle a rare situation involving the inability to reach ID.me's server.
*/
    IDmeVerifyErrorCodeVerificationDidFailToFetchUserProfile,
    
    /// Erorr occurs if user succesfully verified their group affiliation, but decided to deny access to your app at the end of the OAuth flow.
    IDmeVerifyErrorCodeVerificationWasDeniedByUser,
    
    /// Error occurs if user exits modal navigation controller before OAuth flow could complete.
    IDmeVerifyErrorCodeVerificationWasCanceledByUser
};

/// This typedef differentiates the various types of group affiliations and their related web verifications.
typedef NS_ENUM(NSUInteger, IDmeVerifyAffiliationType)
{
    /// @b TroopID Verification
    IDmeVerifyAffiliationTypeMilitary = 1,
    
    /// @b StudentID Verification
    IDmeVerifyAffiliationTypeStudent,
    
    /// @b FirstResponderID Verification
    IDmeVerifyAffiliationTypeResponder
};

@interface IDmeVerify : NSObject

/// The serial-key used to initialize the SDK.
@property (nonatomic, copy, readonly) NSString *serialKey;

/*!
 * The singleton method for the IDmeVerify class.
 *
 * @return A singleton instance of IDmeVerify.
 */
+ (instancetype)sharedInstance;

/*!
 * This method initializes the ID.me Verify SDK. As such, it must be called
 * before any other method in the SDK. Contact @b mobile\@id.me to request a serial key.
 *
 * @param serialKey A serial key that is provided by ID.me.
 */
- (void)startWithSerialKey:(NSString *)serialKey;

/*!
 * This method initializes and launches the credential scanner, which is a modal viewController that is loaded on top of a top-level viewController. The scanner works best in landscape mode.
 *
 * @param externalViewController The viewController which will present the credential scanner model viewController.
 * @param credentialTypes One or more credential types defined in the @b IDmeVerifyDetectCredentialTypes options enumerator.
 * @param scanResults A block that returns the parsed results of the scanned credential and an NSError object.
 */
- (void)scanCredentialInViewController:(UIViewController *)externalViewController
                    forCredentialTypes:(IDmeVerifyDetectCredentialTypes)credentialTypes
                           withResults:(IDmeVerifyScanResults)scanResults;

/*!
 * This method launches a webView initialized inside of navigationController, which is a modal viewController that is loaded on top of a top-level viewController.
 *
 * @param externalViewController The viewController which will present the modal navigationController.
 * @param clientID The @c clientID provided by ID.me when registering the app at http://developer.sandbox.id.me or http://developer.id.me
 * @param redirectURI The @c redirectURI provided to ID.me when registering your app at http://develpper.sandbox.id.me or http://developer.id.me
 * @param affiliationType The type of group verficiation that should be presented. Check the @c IDmeVerifyAffiliationType typedef for more details.
 * @param sandboxMode While developing the app, set this value to @c YES. The test data values provided by ID.me will be accessible via the sandbox API routes. Before pushing your application live and setting this value to @c NO, make sure your app is first registered with ID.me's live/production server by filling out a request at http://developer.sandbox.id.me.
 * @param verificationResults A block that returns an NSDictionary object and an NSError object. The verified user's profile is stored in an NSDictionary object as JSON data. If no data was returned, or an error occured, NSDictionary is nil and NSError returns an error code and localized description of the specific error that occured.
 */
- (void)verifyUserInViewController:(UIViewController *)externalViewController
                      withClientID:(NSString *)clientID
                       redirectURI:(NSString *)redirectURI
                   affiliationType:(IDmeVerifyAffiliationType)affiliationType
                     inSandboxMode:(BOOL)sandboxMode
                       withResults:(IDmeVerifyVerificationResults)verificationResults;

@end
